/**	
*   @file	OArc.cpp
*	@author	Mathieu Delalandre, Sebastien Perin
*	@date	05/2004	
*/

#include <string>
#include <math.h>

using namespace std;

#include <GOMLib\OGraphic.hpp>
#include <GOMLib\OPoint.hpp>
#include <GOMLib\OLine.hpp>
#include <GOMLib\PMath.hpp>

#include <GOMLib\OArc.hpp>


/**
*  Constructor
*/
OArc::OArc()
{
	isT = true;
}


/**
*  Constructor
*/
OArc::OArc(OPoint & b, OPoint & c, OPoint & e,bool isT = true)
{
	this->c = c;
	this->b = b;
	this->e = e;
	this->isT = isT;
}


/**
*  Destructor
*/
OArc::~OArc()
{
}


/**
*  Get radius
*/
double OArc::getRadius()
{
	return c.euclid(b);
}


/**
*  Get direct angle
*/
double OArc::getDAngle()
{
	double r = getRadius();
	double l = b.euclid(e);
	return acos(1 - ((pow(l,2)/pow(r,2))*0.5));
}


/**
*  Is large
*/
bool OArc::isLarge()
{
	OLine l1(b,c),l2(c,e);
	double a = l1.getAngle(l2);
	if(a>pi)
		{
		if(isT)
			return false;
		else
			return true;
		}
	else
		{
		if(isT)
			return true;
		else
			return false;
		}
}


/**
*  Get direct angle
*/
double OArc::getAngle()
{
	double a = getDAngle();
	if(isLarge())
		return 2*pi - a;
	else
		return a;
}


/**
*  Is sweep
*/
bool OArc::isSweep()
{
	OLine be(b,e),ec(e,c);
	if(be.getAngle(ec)<pi)
		return true;
	else
		return false;
}


/**
*  Get length
*/
double OArc::getLength()
{
	return getRadius()*getAngle();
}


/**
*  Check if b-c and e-c are equals,
*  then arc object corresponds to a correct model
*/
bool OArc::check()
{
	if(nearest(c.euclid(b))==nearest(c.euclid(e)))
		return true;
	return false;
}


/**
*  GOM output
*/
void OArc::gomo(ostream & os)
{
	os	<< getName() << " " << isT << " " << endl;
	b.gomo(os);
	c.gomo(os);
	e.gomo(os);
}


/**
*  GOM input
*/
void OArc::gomi(istream & is)
{
	is >> isT;
	string ptn;
	is >> ptn; b.gomi(is);
	is >> ptn; c.gomi(is);
	is >> ptn; e.gomi(is);
}


/**
*  XML output
*/
void OArc::xmlo(ostream & os)
{
	os	<< "<" << getName() ;
	if(isValid())
		{
		os	<< " radius=\"" << getRadius() << "\"" << " angle=\"" << getAngle() << "\"" << " length=\"" << getLength() << "\"";
		os	<< " trigo=\"" << isT << "\"" << " large=\"" << isLarge() << "\"" << " sweep=\"" << isSweep() << "\"";
		}
	else
		{
		os	<< " radius=\"0\" angle=\"0\" length=\"0\"";
		os	<< " trigo=\"0\" large=\"0\" sweep=\"0\"";
		}
	os	<< ">" << endl ;
	b.xmlo(os);
	c.xmlo(os);
	e.xmlo(os);
	os << "</" << getName() << ">" << endl;
}


/**
*  SVG output
*/
void OArc::svgo(ostream & os)
{
	if(isValid())
		{
		double r = getRadius();

		os	<< "<path";
		os	<< " d=\"" 
			<< "M " << b.getX() << "," << b.getY()
			<< " a" << r << "," << r << " 0 " << isLarge() << "," << isSweep() 
			<< " " << b.getDX(e) << "," << b.getDY(e) 
			<< "\"";
		os << " style=\"fill:none; stroke:yellow; stroke-width:1\"";
		os << "/>" << endl;
		}
}
