/**	
*   @file	OGraph.cpp
*	@author	Mathieu Delalandre
*	@date	05/2004	
*/

#include <iostream>
#include <deque>
#include <GTL\graph.h>

using namespace std;

#include <GOMLib\OGraphic.hpp>
#include <GOMLib\AGraphic.hpp>
#include <GOMLib\OGraphicImpl.hpp>
#include <GOMLib\AGraphicImpl.hpp>
#include <GOMLib\OHL.hpp>

#include <GOMLib\OGraph.hpp>

/**
* Clone
*/
OGraphic* OGraph::clone() const
{
	OGraph *g = new OGraph();
	
	int i; 
	
	OGraphic *o;
	graph::node_iterator nit = og.nodes_begin();
	for(i=0;i<og.number_of_nodes();i++)
		{
		o = nl[*nit]->clone(); 
		g->newNode(o); 
		
		nit++;
		}

	graph::edge_iterator eit = og.edges_begin();
	AGraphic *a; int n1,n2;
	for(i=0;i<og.number_of_edges();i++)
		{
		n1 = rangeRead(eit->source());
		n2 = rangeRead(eit->target());
		a = el[*eit]->clone(); 
		g->newEdge(n1,n2,a); 
		
		eit++;
		}
	
	return g;
}


/**
*  Constructor
*/
OGraph::OGraph()
{
	OGraphic* o = new OGraphicImpl(); 
	nl.init(og,o);
	og.make_directed();
}


/**
*  Constructor
*  true graph is directed
*  false graph is undirected
*/
OGraph::OGraph(bool d)
{
	OGraphic* o = new OGraphicImpl(); 
	nl.init(og,o);
	if(d)
		og.make_directed();
	else
		og.make_undirected();
}


/**
*  Destructor
*/
OGraph::~OGraph()
{
	int i;
	
	OGraphic *op;
	graph::node_iterator nit = og.nodes_begin();
	for(i=0;i<og.number_of_nodes();i++)
		{
		op = nl[*nit]; op->~OGraphic();
		nit++;
		}
	
	AGraphic *ap;
	graph::edge_iterator eit = og.edges_begin();
	for(i=0;i<og.number_of_edges();i++)
		{
		ap = el[*eit]; ap->~AGraphic();
		eit++;
		}

	og.clear();
}


/**
*  Read & parse OGraphic array, and push back into the parameter array  'a'
*  the function search into the contenair objects (Graph and OHL) as a tree search function
*/
deque<OGraphic*> OGraph::readOGraphicArray(string & name)
{
	deque<OGraphic*> a; 
	readOGraphicArray(name,a);
	return a;
}


/**
*  Read & parse OGraphic array, and push back into the parameter array  'a'
*  the function search into the contenair objects (Graph and OHL) as a tree search function
*/
void OGraph::readOGraphicArray(string & name,deque<OGraphic*> & a)
	{
	OGraphic* o; string oname;
	for(int i=0;i<og.number_of_nodes();i++)
		{
		o = objectRead(i); oname = o->getName();
		if(oname == name)
			a.push_back(o);
		if(oname == "OHL")
			dynamic_cast<OHL*>(o)->readOGraphicArray(name,a);
		else if(oname == "OGraph")
			dynamic_cast<OGraph*>(o)->readOGraphicArray(name,a);
		}
	}

	
/**
*  Write & parse the OGraphic array according to an object name
*  the function search into the container objects (Graph and OHL) as a tree search function
*/
void OGraph::writeOGraphicArray(string & name, deque<OGraphic*> & a)
{
	writeOGraphicArray(name,a,0); 
}


/**
*  Write & parse the OGraphic array according to an object name
*  the function search into the container objects (Graph and OHL) as a tree search function
*  Warning: use it with k=0
*/
int OGraph::writeOGraphicArray(string & name, deque<OGraphic*> & a, int k)
{
	OGraphic* o; string oname;
	for(int i=og.number_of_nodes()-1;i>=0;i--)
		{
		o = objectRead(i); oname = o->getName();
		if(oname == "OHL")
			k = dynamic_cast<OHL*>(o)->writeOGraphicArray(name,a,k);
		else if(oname == "OGraph")
			k = dynamic_cast<OGraph*>(o)->writeOGraphicArray(name,a,k);
		if(oname == name)
			if(k < og.number_of_nodes())
				{ 
				if(o != a[a.size()-1-k])
					o->~OGraphic();
				objectWrite(a[a.size()-1-k],i);
				k++;
				}
		}
	return k;
}


/**
*  GOM output
*/
void OGraph::gomo(ostream & os)
{
	int i; 

	os	<< getName() 
		<< " " << og.number_of_nodes() 
		<< " " << og.number_of_edges() 
		<< " " << og.is_directed()
		<< endl;
	
	graph::node_iterator nit = og.nodes_begin(); OGraphic *op;
	for(i=0;i<og.number_of_nodes();i++)
		{
		op = nl[*nit]; 
		op->gomo(os);
		nit++; 
		}

	graph::edge_iterator eit = og.edges_begin(); 
	node n1,n2; AGraphic *ap;
	for(i=0;i<og.number_of_edges();i++)
		{
		n1 = eit->source(); n2 = eit->target();
		os << rangeRead(n1) << " " << rangeRead(n2) << endl;
		ap = el[*eit]; ap->gomo(os);
		eit++; 
		}
}


/**
*  GOM input
*/
void OGraph::gomi(istream & is)
{
	int i,os,es,d;
	string name;

	is >> os; is >> es; is >> d;
	
	if(d==0)
		og.make_undirected();
	
	OGraphic *o;
	for(i=0;i<os;i++)
		{
		is >> name;
		o = oCreate(name);
		o->gomi(is);
		newNode(o);
		}
	
	AGraphic *a; AGraphicImpl imp;
	int n1, n2;
	for(i=0;i<es;i++)
		{
		is >> n1; is >> n2;
		is >> name;
		a = imp.aCreate(name);
		a->gomi(is);
		
		newEdge(n1,n2,a);
		}
}


/**
*  XML output
*/
void OGraph::xmlo(ostream & os)
{
	int i;
	os << "<" << getName()  << " directed=\"" << og.is_directed() << "\">" << endl;
	
	os << "<nodes" << " size=\"" << og.number_of_nodes() << "\">" << endl;
	graph::node_iterator nit = og.nodes_begin(); OGraphic *op;
	for(i=0;i<og.number_of_nodes();i++)
		{
		os << "<node id=\"" << i << "\">" << endl;
		op = nl[*nit]; 
		op->xmlo(os);
		nit++; 
		os << "</node>" << endl;
		}
	os << "</nodes>" << endl;

	os << "<edges" << " size=\"" << og.number_of_edges() << "\">" << endl;
	graph::edge_iterator eit = og.edges_begin(); 
	node n1,n2; AGraphic *ap;
	for(i=0;i<og.number_of_edges();i++)
		{
		n1 = eit->source(); n2 = eit->target();
		os	<< "<edge"
			<< " id=\"" << i << "\"" 
			<< " source=\"" << rangeRead(n1) << "\"" 
			<< " target=\"" << rangeRead(n2) << "\""
			<< ">" << endl;
		ap = el[*eit]; ap->xmlo(os);
		eit++; 
		os << "</edge>" << endl;
		}
	os << "</edges>" << endl;

	os << "</" << getName() << ">" << endl; 
}


/**
*  SVG output
*/
void OGraph::svgo(ostream & os)
{
	graph::node_iterator nit = og.nodes_begin(); OGraphic *op;
	for(int i=0;i<og.number_of_nodes();i++)
		{
		op = nl[*nit]; 
		op->svgo(os);
		nit++; 
		}
}
