/**	
*   @file	OHL.cpp
*	@author	Mathieu Delalandre, Sebastien Perin
*	@date	06/2004	
*/

#include <iostream>
#include <string>
#include <deque>
#include <GTL\graph.h>

using namespace std;

#include <GOMLib\OGraphic.hpp>
#include <GOMLib\AGraphic.hpp>
#include <GOMLib\AGraphicImpl.hpp>
#include <GOMLib\OGraphicImpl.hpp>
#include <GOMLib\OPoint.hpp>
#include <GOMLib\OLine.hpp>
#include <GOMLib\OGraph.hpp>
#include <GOMLib\OHL.hpp>


/**
* Clone
*/
OGraphic* OHL::clone() const
{
	OHL *o = new OHL();
	int i;
	for(i=0;i<oa.size();i++)
		o->oa.push_back(oa[i]->clone());
	for(i=0;i<aa.size();i++)
		o->aa.push_back(aa[i]->clone());
	return o;
}


/**
* Constructor
*/
OHL::OHL()
{
}


/**
* Destructor
*/
OHL::~OHL()
{
	oClear();
	aClear();
}


/**
* Object array clear
*/
void OHL::oClear()
{
	for(int i=0;i<oa.size();i++)
		oa[i]->~OGraphic();
	oa.clear();
}


/**
* Attribute array clear
*/
void OHL::aClear()
{
	for(int i=0;i<aa.size();i++)
		aa[i]->~AGraphic();
	aa.clear();
}


/**
* Return first object name or "none"
*/
string OHL::getFOName()
{
	if(oa.size()>=1)
		return oa[0]->getName();
	else
		return "none";
}

	
/**
* Return first attribute name or "none"
*/
string OHL::getFAName()
{
	if(aa.size()>=1)
		return aa[0]->getName();
	else
		return "none";
}

	
/**
* Return true if OHL is composed
* of same objects
*/
bool OHL::sameObject()
{
	string name = getFOName();
	
	if(name != "none")
		{
		for(int i=0;i<oa.size();i++)
			if(name != oa[i]->getName())
				return false;
		}
	else
		return false;
	
	return true;
}


/**
* Return true if OHL is composed
* of same given object name
*/
bool OHL::sameObject(string & name)
{
	if((sameObject())&&(getFOName()==name))
		return true;
	return false;
}


/**
* Return true if OHL is composed
* of same given objects names
*/
bool OHL::sameObject(string & name1, string & name2)
{
	if((sameStringObject(name1,name2))&&((getFOName()==name1)||(getFOName()==name2)))
		return true;
	return false;
}


bool OHL::sameStringObject(string & name1, string & name2)
{	
	string name = getFOName();

	if(name1 == name)
	{
		int i = 0;
		while(name1 == oa[i]->getName())
			i++;
		for(int j=i;j<oa.size();j++)
			if(name2 != oa[j]->getName())
				return false;
	}
	else
		if(name2 == name)
		{
			int i = 0;
			while(name2 == oa[i]->getName())
				i++;
			for(int j=i;j<oa.size();j++)
				if(name1 != oa[j]->getName())
					return false;
		}
		else
			return false;
	
	return true;
}


/**
* Return true if OHL is composed
* of same attribute
*/
bool OHL::sameAttribute()
{
	string name = getFAName();
	
	if(name != "none")
		{
		for(int i=0;i<aa.size();i++)
			if(name != aa[i]->getName())
				return false;
		}
	else
		return false;
	
	return true;
}


/**
* Return true if OHL is composed
* of a same given attribute name
*/
bool OHL::sameAttribute(string & name)
{
	if((sameAttribute())&&(getFAName()==name))
		return true;
	return false;
}


/**
*  Read & parse OGraphic array according to an object name
*  the function search into the container objects (Graph and OHL) as a tree search function
*/
deque<OGraphic*> OHL::readOGraphicArray(string & name)
{
	deque<OGraphic*> a; 
	readOGraphicArray(name,a);
	return a;
}


/**
*  Read & parse OGraphic array according to an object name
*  the function search into the container objects (Graph and OHL) as a tree search function
*/
void OHL::readOGraphicArray(string & name,deque<OGraphic*> & a)
	{
	string oname;
	for(int i=0;i<oa.size();i++)
		{
		oname = oa[i]->getName();
		if(oname == name)
			a.push_back(oa[i]);
		if(oname == "OHL")
			dynamic_cast<OHL*>(oa[i])->readOGraphicArray(name,a);
		else if(oname == "OGraph")
			dynamic_cast<OGraph*>(oa[i])->readOGraphicArray(name,a);
		}
	}

	
/**
*	Write & erase the OGraphic array
*/
void OHL::writeOGraphicArray(deque<OGraphic*> & a)
{
	oClear();
	addOGraphicArray(a);
}


/**
*  Add (writting) the OGraphic array
*/
void OHL::addOGraphicArray(deque<OGraphic*> & a)
{
	for(int i=0;i<a.size();i++)
		oa.push_back(a[i]);
}


/**
*  Write & parse the OGraphic array according to an object name
*  the function search into the container objects (Graph and OHL) as a tree search function
*/
void OHL::writeOGraphicArray(string & name, deque<OGraphic*> & a)
{ 
	writeOGraphicArray(name,a,0); 
}


/**
*  Write & parse the OGraphic array according to an object name
*  the function search into the container objects (Graph and OHL) as a tree search function
*  Warning: use it with k=0
*/
int OHL::writeOGraphicArray(string & name, deque<OGraphic*> & a, int k)
{
	string oname;
	for(int i=oa.size()-1;i>=0;i--)
		{
		oname = oa[i]->getName();
		if(oname == "OHL")
			k = dynamic_cast<OHL*>(oa[i])->writeOGraphicArray(name,a,k);
		if(oname == name)
			if(k < a.size())
				{ 
				if(oa[i] != a[a.size()-1-k])
					oa[i]->~OGraphic(); 
				oa[i] = a[a.size()-1-k]; 
				k++; 
				}
		}
	return k;
}


/**
*  GOM output
*/
void OHL::gomo(ostream & os)
{
	int i;
	os << getName() << " " << oa.size() << " " << aa.size() << endl;
	for(i=0;i<oa.size();i++)
		oa[i]->gomo(os);
	for(i=0;i<aa.size();i++)
		aa[i]->gomo(os);
}


/**
*  GOM input
*/
void OHL::gomi(istream & is)
{
	int i,os,as;
	string name;

	is >> os; is >> as;
	
	OGraphic *o;
	for(i=0;i<os;i++)
		{
		is >> name;
		o = oCreate(name);
		o->gomi(is);
		oa.push_back(o);
		}
	
	AGraphic *a; AGraphicImpl d;
	for(i=0;i<as;i++)
		{
		is >> name;
		a = d.aCreate(name);
		a->gomi(is);
		aa.push_back(a);
		}
}


/**
*  XML output
*/
void OHL::xmlo(ostream & os)
{
	int i;
	os	<< "<" << getName() << ">" << endl;
	
	os	<< "<oa size=\"" << oa.size() << "\" same=\"" << sameObject() << "\">" << endl;
	for(i=0;i<oa.size();i++)
		{
		os << "<o id=\"" << i << "\">" << endl;
		oa[i]->xmlo(os);
		os << "</o>" << endl;
		}
	os	<< "</oa>" << endl;
	
	os	<< "<aa size=\"" << aa.size() << "\" same=\"" << sameAttribute() << "\">" << endl;
	for(i=0;i<aa.size();i++)
		{
		os << "<a id=\"" << i << "\">" << endl;
		aa[i]->xmlo(os);
		os << "</a>" << endl;
		}
	os	<< "</aa>" << endl;

	os << "</" << getName() << ">" << endl; 
}


/**
*  SVG output
*/
void OHL::svgo(ostream & os)
{
	bool s = sameObject();
	string name = getFOName();

	if((s)&&(name == "OPoint"))
		svgPoint(os);
	else if((s)&&(name == "OLine"))
		svgLine(os);
	else
		for(int i=0;i<oa.size();i++)
			oa[i]->svgo(os);
}


/**
* SVG output of OPoint OHL
* for reducing svg code size
*/
void OHL::svgPoint(ostream & os)
{
	bool ic = pointsClosed();
	
	if(ic)
		os	<< "<polygon style=\"fill:none; stroke:blue; stroke-width:1\"";
	else
		os	<< "<polyline style=\"fill:none; stroke:green; stroke-width:1\"";

	OPoint *p; int size = oa.size();
	os	<< " points=\"";
	for(int i=0;i<size;i++)
		{
		p = dynamic_cast<OPoint*>(oa[i]);
		os << p->getX() << "," << p->getY() << " ";
		}
	if(size==1)
		os << p->getX()+1 << "," << p->getY() << " ";
	os	<<	"\"/>" << endl;
}


/**
* The OHL's points are closed
*/
bool OHL::pointsClosed()
{
	OPoint *p1,*p2; int size = oa.size();
	if(size>=4)
		{
		p1 = dynamic_cast<OPoint*>(oa[0]);
		p2 = dynamic_cast<OPoint*>(oa[size-1]);
		if(p1->isConnected(*p2))
			return true;
		}
	return false;
}


/**
* SVG output of OLine OHL
* for reducing svg code size
*/
/*void OHL::svgLine(ostream & os)
{
	bool ic = linesClosed();
	
	if(ic)
		os	<< "<polygon style=\"fill:none; stroke:blue; stroke-width:1\"";
	else
		os	<< "<polyline style=\"fill:none; stroke:green; stroke-width:1\"";
	
	OLine *l; OPoint b,e;int size = oa.size();
	os	<< " points=\"";
	for(int i=0;i<size;i++)
		{
		l = dynamic_cast<OLine*>(oa[i]);	
		b = l->getBegin();
		os << b.getX() << "," << b.getY() << " ";
		e = l->getEnd();
		os << e.getX() << "," << e.getY() << " ";
		}
	os	<<	"\"/>" << endl;
}*/

void OHL::svgLine(ostream & os)
{
	bool ic1 = linesClosed();
	bool ic2 = linesConnected();
	
	if((ic1 == false) && (ic2 == false))
		{ svgOrdinaryLines(os); return; }
	else
	{
		if(ic1)
			os	<< "<polygon style=\"fill:none; stroke:blue; stroke-width:1\"";
		else 
			os	<< "<polyline style=\"fill:none; stroke:red; stroke-width:1\"";
	}


	/*if(ic1)
		os	<< "<polygon style=\"fill:none; stroke:blue; stroke-width:1\"";
	else
	{
		if(ic2)
			os	<< "<polyline style=\"fill:none; stroke:red; stroke-width:1\"";
		else 
			svgOrdinaryLines(os);
	}*/

	
	OLine *l; OPoint b,e;int size = oa.size();
	os	<< " points=\"";
	for(int i=0;i<size;i++)
		{
		l = dynamic_cast<OLine*>(oa[i]);	
		b = l->getBegin();
		os << b.getX() << "," << b.getY() << " ";
		e = l->getEnd();
		os << e.getX() << "," << e.getY() << " ";
		}
	os	<<	"\"/>" << endl;
}


/**
* The OHL's lines are closed
*/
bool OHL::linesClosed()
{
	OLine *l; OPoint b,e;int size = oa.size();
	if(size>=3)
		{
		l = dynamic_cast<OLine*>(oa[0]); b = l->getBegin();
		l = dynamic_cast<OLine*>(oa[size-1]); e = l->getBegin();
		if(b.isConnected(e))
			return true;
		}
	return false;
}


bool OHL::linesConnected()
{
	int i=0;
	OLine *l; OPoint b,e;int size = oa.size();
	while((i<size) && (size !=0))
		{
		l = dynamic_cast<OLine*>(oa[i]); b = l->getBegin();
		l = dynamic_cast<OLine*>(oa[i+1]); e = l->getBegin();
		if(b.isConnected(e))
			i++;
		else return false;
		}
	return true;
}

void OHL::svgOrdinaryLines(ostream & os)
{
	OLine *l; OPoint b,e;int size = oa.size();
	for(int i=0;i<size;i++)
		{
		l = dynamic_cast<OLine*>(oa[i]);
		l->svgo(os);
	}
}