/**	
*   @file	OPoint.cpp
*	@author	Mathieu Delalandre, Sebastien Perin
*	@date	05/2004	
*/

#include <iostream>
#include <string>
#include <math.h>

using namespace std;

#include <GOMLib\OGraphic.hpp>
#include <GOMLib\PMath.hpp>

#include <GOMLib\OPoint.hpp>


/**
*  Constructor
*/
OPoint::OPoint()
{
	x = y = -1;
}


/**
*  Constructor
*/
OPoint::OPoint(int x, int y)
{
	this->x = x;
	this->y = y;
}


/**
*  Destructor
*/
OPoint::~OPoint()
{
}


/**
*  Get the p(x) - this(x) delta x
*/
int OPoint::getDX(OPoint & p)
	{return p.getX()-x;}


/**
*  Get the p(y) - this(y) delta y 
*/
int OPoint::getDY(OPoint & p)
	{return p.getY()-y;}


/**
*  Euclid
*/
double OPoint::euclid(OPoint & p)
{ 
	return sqrt(pow(x-p.getX(),2) + pow(y-p.getY(),2)); 
}


/**
*  Move
*/
void OPoint::move(int dx,int dy)
{ 
	x += dx ;
	y += dy ; 
}


/**
*  Scalar
*/
void OPoint::scalar(double a)
{ 
	OPoint c(0,0);
	double l = euclid(c);
	x = nearest(l*cos(a));
	y = y - nearest(l*sin(a));
}


/**
*  Get following according 
*  to the Freeman direction (0 to 7)
*/
OPoint OPoint::getFollowing(int d)
{ 
	OPoint p;
	switch(d)
		{
		case 0:
			p.set(x+1,y);
			break;
		case 1:
			p.set(x+1,y-1);
			break;
		case 2:
			p.set(x,y-1);
			break;
		case 3:
			p.set(x-1,y-1);
			break;
		case 4:
			p.set(x-1,y);
			break;
		case 5:
			p.set(x-1,y+1);
			break;
		case 6:
			p.set(x,y+1);
			break;
		case 7:
			p.set(x+1,y+1);
			break;
		}
	return p;
}


/**
* Is connected
*/
bool OPoint::isConnected(OPoint & p)
{ 
	if	(
		(abs(x-p.getX())<=1)
		&&
		(abs(y-p.getY())<=1)
		)
		return true;
	return false;
}


/**
*  Get the Freeman direction to move 
*  to parameter point, or return -1
*/
int OPoint::getDirection(OPoint & p)
{ 
	int dx = p.getX()-x, dy = p.getY()-y;
	
	if((dx==1)&&(dy==0))
		return 0;
	else if((dx==1)&&(dy==-1))
		return 1;
	else if((dx==0)&&(dy==-1))
		return 2;
	else if((dx==-1)&&(dy==-1))
		return 3;	
	else if((dx==-1)&&(dy==0))
		return 4;
	else if((dx==-1)&&(dy==+1))
		return 5;
	else if((dx==0)&&(dy==+1))
		return 6;
	else if((dx==+1)&&(dy==+1))
		return 7;
		
	return -1;
}


/**
*  == operator
*/
bool OPoint::operator == (OPoint & p)
{
	if((x == p.getX())&&(y == p.getY()))
		return true;
	return false;
}


/**
*  != operator
*/
bool OPoint::operator != (OPoint & p)
{
	if((x != p.getX())||(y != p.getY()))
		return true;
	return false;
}


/**
*  + operator
*/
OPoint OPoint::operator + (OPoint & p)
{
	int xr = x + p.getX();
	int yr = y + p.getY();
	OPoint r(xr,yr);
	return r;
}


/**
*  - operator
*/
OPoint OPoint::operator - (OPoint & p)
{
	int xr = x - p.getX();
	int yr = y - p.getY();
	OPoint r(xr,yr);
	return r;
}


/**
*  / operator
*/
OPoint OPoint::operator / (int v)
{
	int xr = x / v;
	int yr = y / v;
	OPoint r(xr,yr);
	return r;
}


/**
*  * operator
*/
OPoint OPoint::operator * (int v)
{
	int xr = x * v;
	int yr = y * v;
	OPoint r(xr,yr);
	return r;
}


/**
*  GOM output
*/
void OPoint::gomo(ostream & os)
{
	os << getName() << " " << x << " " << y << endl;
}


/**
*  GOM input
*/
void OPoint::gomi(istream & is)
{
	is >> x;
	is >> y;
}


/**
*  XML output
*/
void OPoint::xmlo(ostream & os)
{
	os	<< "<" << getName() << " x=\"" << x <<"\" y=\"" << y << "\"/>" << endl;  
}


/**
*  SVG output
*/
void OPoint::svgo(ostream & os)
{
	if(isValid())
		os	<<	"<line"
			<<	" x1=\"" << x << "\" y1=\"" << y << "\""
			<<	" x2=\"" << x+1 << "\" y2=\"" << y << "\""
			<<	" style=\"fill:none; stroke:blue; stroke-width:1\""
			<<	"/>" << endl ;
}

