/**	
*   @file	OVPoint.cpp
*	@author	Mathieu Delalandre
*	@date	06/2004	
*/

#include <iostream>
#include <string>
#include <math.h>

using namespace std;

#include <GOMLib\OGraphic.hpp>
#include <GOMLib\OPoint.hpp>
#include <GOMLib\PMath.hpp>

#include <GOMLib\OVPoint.hpp>


/**
* Constructor
*/
OVPoint::OVPoint()
{
	this->x = -1;
	this->y = -1;
}


/**
* Constructor
*/
OVPoint::OVPoint(double x,double y)
{
	this->x = x;
	this->y = y;
}


/**
* Constructor
*/
OVPoint::~OVPoint()
{
}


/**
*  euclid
*/
double OVPoint::euclid(OVPoint & p)
{ 
	return sqrt(pow(x-p.getX(),2) + pow(y-p.getY(),2)); 
}


/**
*  move
*/
void OVPoint::move(double dx,double dy)
{ 
	x += dx ;
	y += dy ; 
}


/**
*  scalar
*/
void OVPoint::scalar(double a)
{ 
	OVPoint c(0,0);
	double l = euclid(c);
	x = l*cos(a);
	y = y - l*sin(a);
}


/**
*  Is valid
*/
bool OVPoint::isValid()
{ 
	if((x>=0)&&(y>=0))
		return true;
	return false;
}


/**
*  valid
*/
OPoint OVPoint::getOPoint()
{ 
	return OPoint(nearest(x),nearest(y));
}


/**
*  == operator
*/
bool OVPoint::operator == (OVPoint & p)
{
	if((x == p.getX())&&(y == p.getY()))
		return true;
	return false;
}


/**
*  != operator
*/
bool OVPoint::operator != (OVPoint & p)
{
	if((x != p.getX())||(y != p.getY()))
		return true;
	return false;
}


/**
*  + operator
*/
OVPoint OVPoint::operator + (OVPoint & p)
{
	double xr = x + p.getX();
	double yr = y + p.getY();
	OVPoint r(xr,yr);
	return r;
}


/**
*  - operator
*/
OVPoint OVPoint::operator - (OVPoint & p)
{
	double xr = x - p.getX();
	double yr = y - p.getY();
	OVPoint r(xr,yr);
	return r;
}


/**
*  / operator
*/
OVPoint OVPoint::operator / (int v)
{
	double xr = x / v;
	double yr = y / v;
	OVPoint r(xr,yr);
	return r;
}


/**
*  * operator
*/
OVPoint OVPoint::operator * (int v)
{
	double xr = x * v;
	double yr = y * v;
	OVPoint r(xr,yr);
	return r;
}


/**
*  GOM output
*/
void OVPoint::gomo(ostream & os)
{
	os << getName() << " " << x << " " << y << endl;
}


/**
*  GOM input
*/
void OVPoint::gomi(istream & is)
{
	is >> x;
	is >> y;
}


/**
*  XML output
*/
void OVPoint::xmlo(ostream & os)
{
	os	<< "<" << getName() << " x=\"" << x <<"\" y=\"" << y << "\"/>" << endl;  
}


/**
*  SVG output
*/
void OVPoint::svgo(ostream & os)
{
	if(isValid())
		{
		int xs = nearest(x), ys = nearest(y);
		os	<<	"<line"
			<<	" x1=\"" << xs << "\" y1=\"" << ys << "\""
			<<	" x2=\"" << xs+1 << "\" y2=\"" << ys << "\""
			<<	" style=\"fill:none; stroke:blue; stroke-width:1\""
			<<	"/>" << endl ;
		}
}

