/**	
*   @file	OHL.hpp
*	@author	Mathieu Delalandre, Sebastien Perin
*	@date	06/2004	
*/


/**
*  OHL object
*/
class OHL : public OGraphic
{
	protected:	
		void svgPoint(ostream &);
		bool pointsClosed();
		void svgLine(ostream &);
		bool linesClosed();

		bool linesConnected();
		void svgOrdinaryLines(ostream & os);


		
		deque <OGraphic*> oa;	// object array
		deque <AGraphic*> aa;	// attribute array
	
	public:
		OGraphic* clone() const;
		inline OGraphic* create() const;
		OHL();
		~OHL();
			
		void oClear();
		void aClear();
		
		inline int osize();
		inline void newObject();
		inline void newObject(OGraphic*);
		inline void deleteObject(int);
		inline OGraphic* objectRead(int);
		inline int rangeRead(OGraphic*);
		inline void objectWrite(OGraphic*,int);
		inline void objectPush(OGraphic*,int);
		inline void objectInsert(OGraphic*,int);
		
		inline int asize();
		inline void newAttribute();
		inline void newAttribute(AGraphic*);
		inline void deleteAttribute(int);
		inline AGraphic* attributeRead(int);
		inline int rangeRead(AGraphic*);
		inline void attributeWrite(AGraphic*,int);
		inline void attributePush(AGraphic*,int);
		inline void attributeInsert(AGraphic*,int);
		
		string getFOName();
		string getFAName();
		bool sameObject();
		bool sameObject(string &);
		bool sameAttribute();
		bool sameAttribute(string &);
		bool sameObject(string &, string &); //modification
		bool sameStringObject(string &, string &);
		
		inline deque<OGraphic*> readOGraphicArray();
		deque<OGraphic*> readOGraphicArray(string &);
		void readOGraphicArray(string &,deque<OGraphic*> &);
		
		void writeOGraphicArray(deque<OGraphic*> &);
		void addOGraphicArray(deque<OGraphic*> &);
		void writeOGraphicArray(string &,deque<OGraphic*> &);
		int writeOGraphicArray(string &,deque<OGraphic*> &,int);

		//to be continue .... read/write attribute array

		void gomo(ostream &);
		void gomi(istream &);
		void xmlo(ostream &);
		void svgo(ostream &);
};

/**
*  Create a new OHL object
*/
inline OGraphic* OHL::create() const
	{ return new OHL(); }

/**
*  Object array size
*/
inline int OHL::osize()
	{ return oa.size(); }
/**
*  New object
*/
inline void OHL::newObject()
	{ oa.push_back(new OGraphicImpl()); }
/**
*  New object
*/
inline void OHL::newObject(OGraphic* o)
	{ oa.push_back(o); }
/**
*  Delete object (0 to size-1)
*/
inline void OHL::deleteObject(int p)
	{ if(p<oa.size()) { oa[p]->~OGraphic(); oa.erase(oa.begin()+p);} }
/**
*  Object read (0 to size-1)
*/
inline OGraphic* OHL::objectRead(int p)
	{ return oa.at(p); }
/**
*  Range read
*/
inline int OHL::rangeRead(OGraphic* o)
	{for(int i=0;i<oa.size();i++) if(oa[i]==o) return i; return -1;}
/**
*  Object write (0 to size-1)
*/
inline void OHL::objectWrite(OGraphic* o,int p)
	{ if(p<oa.size()) oa[p] = o; }
/**
*  Object push, delete the p existing object
* (0 to size-1)
*/
inline void OHL::objectPush(OGraphic* o,int p)
	{ if(p<oa.size()) { oa[p]->~OGraphic(); oa[p] = o;} }
/**
*  Object insert
*  (0 to size-1)
*/
inline void OHL::objectInsert(OGraphic* o,int p)
	{ if(p<oa.size()) oa.insert(oa.begin()+p,o); }

/**
*  Attribute array size
*/
inline int OHL::asize()
	{ return aa.size(); }
/**
*  New attribute
*/
inline void OHL::newAttribute()
	{ aa.push_back(new AGraphicImpl()); }
/**
*  New attribute
*/
inline void OHL::newAttribute(AGraphic* a)
	{ aa.push_back(a); }
/**
*  Delete attribute
*  (0 to size-1)
*/
inline void OHL::deleteAttribute(int p)
	{ if(p<aa.size()) { aa[p]->~AGraphic(); aa.erase(aa.begin()+p);} }
/**
*  Attribute read
*  (0 to size-1)
*/
inline AGraphic* OHL::attributeRead(int p)
	{ return aa.at(p); }
/**
*  Range read
*/
inline int OHL::rangeRead(AGraphic* a)
	{for(int i=0;i<aa.size();i++) if(aa[i]==a) return i; return -1;}
/**
*  Attribute write
*  (0 to size-1)
*/
inline void OHL::attributeWrite(AGraphic* a,int p)
	{ if(p<aa.size()) aa[p] = a; }
/**
*  Attribute push, delete the p existing attribute
*  (0 to size-1)
*/
inline void OHL::attributePush(AGraphic* a,int p)
	{ if(p<aa.size()) {aa[p]->~AGraphic(); aa[p] = a;} }
/**
*  Attribute insert
*  (0 to size-1)
*/
inline void OHL::attributeInsert(AGraphic* a,int p)
	{ if(p<aa.size()) aa.insert(aa.begin()+p,a); }
/**
*  Read OGraphic array
*/
deque<OGraphic*> OHL::readOGraphicArray()
	{ return oa; }
