/**	
*   @file	OLine.hpp
*	@author	Mathieu Delalandre, Sbastien Perin
*	@date	05/2004	
*/

/**
*  OLine object
*/
class OLine : public OGraphic 
{
	protected:
		OPoint b,e;

	public:
		inline OGraphic* clone() const;
		inline OGraphic* create() const;
		OLine();
		OLine(OPoint &,OPoint &);
		OLine(int,int,int,int);
		~OLine();

		int getDX();
		int getDY();
		double getLength();
		double matchLength(OLine &);
		bool isOpposite(OLine &);
		void move(int,int,int=1);
		void movec();
		void reverse();
		OLine getReverse();
		OPoint getMiddle();
		bool isConnected(OLine &);
		bool isNear(OLine &);
		double getTangent();
		int getOrigin();
		double getDirection();
		double getDAngle(OLine &);
		double getAngle(OLine &);
		double getNAzimuth();
		double getSAzimuth();
		double getEAzimuth();
		double getWAzimuth();
		bool isParallel(OLine &);
		double matchTeta(OLine &);
		bool isOrthogonal(OLine &);
		bool isColinear(OPoint &);
		bool between(OPoint &);

		bool belong(OPoint &);
		OPoint getIntersection(OLine &);
		bool intersect(OLine &);
		bool overlap(OLine &);
		double euclid(OPoint &);
		double euclid(OLine &);
		
		bool operator == (OLine &);
		bool operator != (OLine &);
		OLine operator / (int);
		OLine operator * (int);

		void gomo(ostream &);
		void gomi(istream &);
		void svgo(ostream &);
		void xmlo(ostream &);
		
		inline bool isValid();
		inline void clear();
		inline void set(OPoint &,OPoint &);
		inline void set(int,int,int,int);
		inline void setBegin(OPoint &);
		inline void setBegin(int,int);
		inline void setEnd(OPoint &);
		inline void setEnd(int,int);
		inline OPoint getBegin();
		inline OPoint getEnd();
};

/**
*  Clone the line object
*/
inline OGraphic* OLine::clone() const
	{ return new OLine(*this); }
/**
*  Create a new line object
*/
inline OGraphic* OLine::create() const
	{ return new OLine(); }

/**
*  Is valid 
*  begin and enb no null
*/
bool OLine::isValid()
	{ if(b.isValid() && e.isValid()) return true; return false; }
/**
*  Clear the begin;end point
*/
inline void OLine::clear()
	{ b.clear(); e.clear();}
/**
*  Set the begin;end point
*/
inline void OLine::set(OPoint & b,OPoint & e)
	{ this->b = b; this->e = e;}
/**
*  Set the x1;y1;x2;y2 begin;end point
*/
inline void OLine::set(int x1,int y1,int x2, int y2)
	{ b.set(x1,y1); e.set(x2,y2);}
/**
*  Set the begin point
*/
inline void OLine::setBegin(OPoint & b)
	{ this->b = b;}
/**
*  Set the x;y begin point
*/
inline void OLine::setBegin(int x,int y)
	{ b.set(x,y);}
/**
*  Set the end point
*/
inline void OLine::setEnd(OPoint & e)
	{ this->e = e;}
/**
*  Set the x;y end point
*/
inline void OLine::setEnd(int x,int y)
	{ e.set(x,y);}

/**
*  Get the begin point
*/
inline OPoint OLine::getBegin()
	{return b;}
/**
*  Get the end point
*/
inline OPoint OLine::getEnd()
	{return e;} 
