/**	
*   @file	OPoint.hpp
*	@author	Mathieu Delalandre, Sebastien Perin
*	@date	05/2004	
*/


/**
*  OPoint object
*/
class OPoint : public OGraphic
{
	protected:
		int x;
		int y;		

	public:
		inline OGraphic* clone() const;
		inline OGraphic* create() const;
		OPoint();
		OPoint(int,int);
		~OPoint();

		int getDX(OPoint &);
		int getDY(OPoint &);
		double euclid(OPoint &);
		void move(int,int);
		void scalar(double);
		OPoint getFollowing(int);
		bool isConnected(OPoint &);
		int getDirection(OPoint &);

		bool operator == (OPoint &);
		bool operator != (OPoint &);
		OPoint operator + (OPoint &);
		OPoint operator - (OPoint &);
		OPoint operator / (int);
		OPoint operator * (int);

		void gomo(ostream &);
		void gomi(istream &);
		void xmlo(ostream &);
		void svgo(ostream &);

		inline bool isValid();
		inline void clear();
		inline void set(int,int);
		inline void setX(int);
		inline void setY(int);
		inline int getX();
		inline int getY();
};

/**
*  Clone the point object
*/
inline OGraphic* OPoint::clone() const
	{ return new OPoint(*this); }
/**
*  Create a new point object
*/
inline OGraphic* OPoint::create() const
	{ return new OPoint(); }


/**
*  Is valid
*  x and y no null
*/
inline bool OPoint::isValid()
	{ if((x>=0)&&(y>=0)) return true; return false;}
/**
*  Clear the x;y values
*/
inline void OPoint::clear()
	{ this->x = -1; this->y = -1; }
/**
*  Set the x;y values
*/
inline void OPoint::set(int x,int y)
	{ this->x = x; this->y = y; }
/**
*  Set the x value
*/
inline void OPoint::setX(int x)
	{ this->x = x;}
/**
*  Set the y value
*/
inline void OPoint::setY(int y)
	{ this->y = y;}
/**
*  Get the x value
*/
inline int OPoint::getX()
	{return x;}
/**
*  Get the y value
*/
inline int OPoint::getY()
	{return y;}
