/**	
*   @file	OPoint.hpp
*	@author	Mathieu Delalandre
*	@date	05/2004	
*/


/**
*  OVPoint object, a virtual point object
*  with double x,y 
*/
class OVPoint : public OGraphic
{
	protected:
		double x;
		double y;

	public:
		inline OGraphic* clone() const;
		inline OGraphic* create() const;

		OVPoint();
		OVPoint(double,double);
		~OVPoint();

		double euclid(OVPoint &);
		void move(double,double);
		void scalar(double);
		bool isValid();
		OPoint getOPoint();

		bool operator == (OVPoint &);
		bool operator != (OVPoint &);
		OVPoint operator + (OVPoint &);
		OVPoint operator - (OVPoint &);
		OVPoint operator / (int);
		OVPoint operator * (int);
	
		void gomo(ostream &);
		void gomi(istream &);
		void xmlo(ostream &);
		void svgo(ostream &);

		inline void clear();
		inline void set(double,double);
		inline void setX(double);
		inline void setY(double);
		inline double getX();
		inline double getY();
		inline double getDX(OVPoint &);
		inline double getDY(OVPoint &);
};

inline OGraphic* OVPoint::clone() const
	{ return new OVPoint(*this); }
inline OGraphic* OVPoint::create() const
	{ return new OVPoint(); }

inline void OVPoint::clear()
	{ this->x = -1; this->y = -1; }
inline void OVPoint::set(double x,double y)
	{ this->x = x; this->y = y; }
inline void OVPoint::setX(double x)
	{ this->x = x;}
inline void OVPoint::setY(double y)
	{ this->y = y;}
inline double OVPoint::getX()
	{return x;}
inline double OVPoint::getY()
	{return y;}
inline double OVPoint::getDX(OVPoint & p)
	{return p.getX()-x;}
inline double OVPoint::getDY(OVPoint & p)
	{return p.getY()-y;}
