/**	\file	IntLib.cpp
*	\author	Mathieu Delalandre
*	\date	05/04
*/

//include
#include <iostream>
#include <fstream>
#include <vector>
#include <deque>

using namespace std;

#include <JNI\jni.h>
#include <LibTiff\tiff.h>
#include <Sys\DL.hpp>
#include <GTL\graph.h>
#include <TILib\DataImage.hpp>
#include <TILib\FileTiff.hpp>
#include <TILib\IntJAIImage.hpp>
#include <GOMLib\GOMLib.hpp>

#include <LGALib\TreatChain.hpp>
#include <LGALib\TreatWDPoly.hpp>
#include <LGALib\TreatMDPoly.hpp>
#include <LGALib\TreatCMatching.hpp>
#include <LGALib\TreatStructuring.hpp>
#include <LGALib\FileName.hpp>

/**
* Image Chaining
*/
void ImageChaining(Image* img,char* rf)
	{
	img->frameInitialisation();
	IChain *ic = new IChain();
	ic->imageChain(img);
	gomWrite(ic->il,ExtAdd(rf,".gom"));
	xmlWrite(ic->il,ExtAdd(rf,".xml"));
	svgWrite(ic->il,ExtAdd(rf,".svg"));
	ic->~IChain();
	}
void ImageChaining(char* imgfile,char* rfile)
	{
	Tiff* f = new Tiff();
	Image* img = f->tiffRead(imgfile);
	ImageChaining(img,rfile);
	img->~Image();
	}
extern "C"
JNIEXPORT void JNICALL Java_psi_ipl_LGALib_FileImageChaining
		(
		JNIEnv	*env, 
		jobject	obj , 
		jstring	imgnf,
		jstring resnf
		)
	{	
	char *imgnf2 = (char *)env->GetStringUTFChars(imgnf,0);
	char *resnf2 = (char *)env->GetStringUTFChars(resnf,0);
	ImageChaining(imgnf2,resnf2);
	}
extern "C"
JNIEXPORT void JNICALL Java_psi_ipl_LGALib_ImageImageChaining
		(
		JNIEnv		*env,
		jobject		obj,
		jintArray	table,
		jint		width,
		jint		heigth,
		jstring		resnf,
		jint		bc
		)
	{	
	Image *img = intArrayToImage(env,table,width,heigth);
	if(bc==0)
		img->inversion();
	char* resnf2 = (char *)env->GetStringUTFChars(resnf,0);
	ImageChaining(img,resnf2);
	img->~Image();
	}
EXPORT void LGALib_ImageChaining
		(
		char*	imgnf,
		char*	resnf
		)
	{
	ImageChaining(imgnf,resnf);
	}
/**
* Wall & Danielson Polygonisation
*/
void WDPolygonisation(char* eFile,double th,char* rFile)
	{
	OHL* o = castOHL(gomRead(eFile));

	if(o != NULL)
		{
		deque<OGraphic*> tp = o->readOGraphicArray((string)"OHL");
		deque<OHL*> hai = castOHLArray(tp); tp.clear();
		
		WDP *wdp = new WDP(); 
		deque<OGraphic*> har = wdp->polygonalise(hai,th);		
		o->writeOGraphicArray((string)"OHL",har);

		gomWrite(o,ExtAdd(rFile,".gom"));
		xmlWrite(o,ExtAdd(rFile,".xml"));
		svgWrite(o,ExtAdd(rFile,".svg"));
		
		wdp->~WDP();
		hai.clear();
		har.clear();
		o->~OHL();
		}
	}
extern "C"
JNIEXPORT void JNICALL Java_psi_ipl_LGALib_WDPolygonisation
		(
		JNIEnv	*env, 
		jobject	obj , 
		jstring	entnf,
		double	th,
		jstring resnf
		)
	{	
	char *entnf2 = (char *)env->GetStringUTFChars(entnf,0);
	char *resnf2 = (char *)env->GetStringUTFChars(resnf,0);
	WDPolygonisation(entnf2,th,resnf2);
	}
EXPORT void LGALib_WDPolygonisation
		(
		char*	entnf,
		double	th,
		char*	resnf
		)
	{
	WDPolygonisation(entnf,th,resnf);
	}
/**
* Maximum distance Polygonisation
*/
void MDPolygonisation(char* eFile,double th,char* rFile)
	{
	OHL* o = castOHL(gomRead(eFile));

	if(o != NULL)
		{
		deque<OGraphic*> tp = o->readOGraphicArray((string)"OHL");
		deque<OHL*> hai = castOHLArray(tp); tp.clear();
		
		MDP *mdp = new MDP(); 
		deque<OGraphic*> har = mdp->polygonalise(hai,th);		
		o->writeOGraphicArray((string)"OHL",har);

		gomWrite(o,ExtAdd(rFile,".gom"));
		xmlWrite(o,ExtAdd(rFile,".xml"));
		svgWrite(o,ExtAdd(rFile,".svg"));
		
		mdp->~MDP();
		hai.clear();
		har.clear();
		o->~OHL();
		}
	}
extern "C"
JNIEXPORT void JNICALL Java_psi_ipl_LGALib_MDPolygonisation
		(
		JNIEnv	*env, 
		jobject	obj , 
		jstring	entnf,
		double	th,
		jstring resnf
		)
	{	
	char *entnf2 = (char *)env->GetStringUTFChars(entnf,0);
	char *resnf2 = (char *)env->GetStringUTFChars(resnf,0);
	MDPolygonisation(entnf2,th,resnf2);
	}
EXPORT void LGALib_MDPolygonisation
		(
		char*	entnf,
		double	th,
		char*	resnf
		)
	{
	MDPolygonisation(entnf,th,resnf);
	}

/**
* Countour Matching
*/
void CMatching(char* eFile,double th,char* rMFile,char* rUFile,char* rAFile)
	{
	OHL* o = castOHL(gomRead(eFile));
	OHL* o2 = castOHL(o->clone()); //modification
	OHL* o3 = new OHL();

	if(o != NULL)
		{
		deque<OGraphic*> tp = o->readOGraphicArray((string)"OHL");
		deque<OHL*> hai = castOHLArray(tp); tp.clear();
		
		CMatch *cm = new CMatch();
		deque<OGraphic*> har = cm->match(hai,th);
		o->writeOGraphicArray((string)"OHL",har);
		o2 = cm->pae; //modification
		//o3->writeOGraphicArray(cm->npa);
		o3 = cm->all;

		gomWrite(o,ExtAdd(rMFile,".gom")); 
		xmlWrite(o,ExtAdd(rMFile,".xml")); 
		svgWrite(o,ExtAdd(rMFile,".svg")); 
		
		gomWrite(o2,ExtAdd(rUFile,".gom")); //modification
		xmlWrite(o2,ExtAdd(rUFile,".xml")); //modification
		svgWrite(o2,ExtAdd(rUFile,".svg")); //modification

		gomWrite(o3,ExtAdd(rAFile,".gom")); 
		xmlWrite(o3,ExtAdd(rAFile,".xml")); 
		svgWrite(o3,ExtAdd(rAFile,".svg"));

		cm->~CMatch();
		hai.clear();
		har.clear();
		o->~OHL();
		o2->~OHL(); //modification
		o3->~OHL(); //modification
		}
	}
extern "C"
JNIEXPORT void JNICALL Java_psi_ipl_LGALib_CMatching
		(
		JNIEnv	*env, 
		jobject	obj , 
		jstring	entnf,
		double	th,
		jstring resnf,
		jstring resnf1,
		jstring resnf4
		)
	{	
	char *entnf2 = (char *)env->GetStringUTFChars(entnf,0);
	char *resnf2 = (char *)env->GetStringUTFChars(resnf,0);
	char *resnf3 = (char *)env->GetStringUTFChars(resnf1,0);
	char *resnf5 = (char *)env->GetStringUTFChars(resnf4,0);
	CMatching(entnf2,th,resnf2,resnf3,resnf5);
	}
EXPORT void LGALib_CMatching
		(
		char*	entnf,
		double	th,
		char*	resnf,
		char*	resnf1,
		char*	resnf4
		)
	{
	CMatching(entnf,th,resnf,resnf1,resnf4);
	}


/**
* Structured Quadrilaterals
*/
void StrucQuad(char* eFile,double th,char* rFile)
	{
	OHL* o = castOHL(gomRead(eFile));

	if(o != NULL)
		{
		deque<OGraphic*> tp = o->readOGraphicArray((string)"OHL");
		deque<OHL*> hai = castOHLArray(tp); tp.clear();
		
		Structured *st = new Structured();
		deque<OGraphic*> har = st->structuring(hai,th);
		o->writeOGraphicArray((string)"OHL",har);

		gomWrite(o,ExtAdd(rFile,".gom")); 
		xmlWrite(o,ExtAdd(rFile,".xml")); 
		svgWrite(o,ExtAdd(rFile,".svg")); 
		
		st->~Structured();
		hai.clear();
		har.clear();
		o->~OHL(); 
		}
	}
extern "C"
JNIEXPORT void JNICALL Java_psi_ipl_LGALib_StrucQuad
		(
		JNIEnv	*env, 
		jobject	obj , 
		jstring	entnf,
		double	th,
		jstring resnf
		)
	{	
	char *entnf2 = (char *)env->GetStringUTFChars(entnf,0);
	char *resnf2 = (char *)env->GetStringUTFChars(resnf,0);
	StrucQuad(entnf2,th,resnf2);
	}
EXPORT void LGALib_StrucQuad
		(
		char*	entnf,
		double	th,
		char*	resnf
		)
	{
	StrucQuad(entnf,th,resnf);
	}

