/**	\file	TreatChain.cpp
*	\author	Mathieu Delalandre
*	\date	05/04
*/

//include

#include <iostream>
#include <fstream>
#include <vector>
#include <deque>

using namespace std;

#include <GTL\graph.h>
#include <TILib\DataImage.hpp>
#include <GOMLib\GOMLib.hpp>

#include <LibTiff\tiff.h>
#include <TILib\FileTiff.hpp>

#include <LGALib\TreatChain.hpp>


/**
* Static member
*/
int IChain::plab = -1;
int IChain::jlab = -2;

/**
* Constructor
*/
IChain::IChain()
	{
	map = new Image();
	il = new OHL();
	jl = new OHL();
	}

/**
* Destructor
*/
IChain::~IChain()
	{
	map->~Image();
	if(il != NULL)
		il->~OHL();
	}

/**
* Main process
*/
void IChain::imageChain(Image* img)
	{
	this->img = img;
	map = img->copy(); 
	
	junctionFind();
	il->newObject(jl);
	
	ep.set(0,0);
	while(entryFind())
		{
		if(!dual)
			{
			cl = new OHL();
			lineChain(true);
			il->newObject(cl);
			}
		else
			{
			cl = new OHL();
			lineChain(true);
			lineChain(false);
			il->newObject(cl);
			}
		}
	
	}

/**
* Find junction point
*/
void IChain::junctionFind()
	{
	OPoint p; int n;
	for(int y=0;y<img->getHeight();y++)
		for(int x=0;x<img->getWidth();x++)
			if(img->pixelRead(x,y)==1)
				{
				p.set(x,y); 
				n = iNCount(p);
				if(n>=3)
					junctionChain(p);	
				}
	}

/**
* Count the neighboors of a given point
* on initial image
*/
int IChain::iNCount(OPoint & p)
	{
	int n=0; OPoint fp; 
	for(int d=0;d<=7;d++)
		{
		fp = p.getFollowing(d);
		if(img->pixelRead(fp.getX(),fp.getY())==1)
			n++;
		}
	return n;
	}

/**
* Chain a given pixel
* 1- store it in the current hlo
* 2- label the image map with junction label
*/
void IChain::junctionChain(OPoint & p)
	{
	map->pixelWrite(p.getX(),p.getY(),jlab);
	OJunction *aj = new OJunction(p.getX(),p.getY());
	jl->newObject(aj);
	}

/**
* Find an entry point if pixel color is black, if it is no labelled,
* and the neigboor number is equal to 0, 1 or 2
* start the find process at the current entry point !
*/
bool IChain::entryFind()
	{
	OPoint p; int n;
	int xd,yd;
	
	xd=ep.getX(), yd=ep.getY();
	for(int y=yd;y<map->getHeight();y++)
		{
		for(int x=xd;x<map->getWidth();x++)
			if(map->pixelRead(x,y)==1)
				{
				p.set(x,y); n = mNCount(p);
				//alone or entry point
				if((n==0)||(n==1))
					{
					ep.set(x,y);
					dual = false;
					return true;
					}
				//chain point
				else if(n==2)
					{
					ep.set(x,y);
					dual = true;
					return true;
					}
				}
		xd=0;
		}
	
	ep.clear(); dual=false;
	return false;
	}

/**
* Count the neighboors of a given point
* on the map image
*/
int IChain::mNCount(OPoint & p)
	{
	int n=0; OPoint fp; 
	for(int d=0;d<=7;d++)
		{
		fp = p.getFollowing(d);
		if(map->pixelRead(fp.getX(),fp.getY())==1)
			n++;
		}
	return n;
	}

/**
* Chain an image's line according the found entry point
* bool s : sens, true normal add, false reverse add
*/
void IChain::lineChain(bool s)
	{
	cp = ep; int type; bool chain=true;
	while(chain)
		{
		type = chainFollow();
		//alone point or extremity case
		if(type==0)
			{
			pointChain(cp,s); 
			chain=false;
			}
		//entry or chain point case
		else if((type==1)||(type==2))
			{
			pointChain(cp,s); 
			cp = np; 
			}
		}
	}

/**
* Find a neighboor point 'np' of current point 'cp'
* return n the number of neighboor 
*/
int IChain::chainFollow()
	{
	int n=0; OPoint fp; np.clear();
	for(int d=0;d<=7;d++)
		{
		fp = cp.getFollowing(d);
		if(map->pixelRead(fp.getX(),fp.getY())==1)
			{ n++; np = fp; }
		}
	return n;
	}

/**
* Chain a given pixel, 
* param sens : true back push, false front push (reverse)
* 1- store it in the current hlo
* 2- label the image map with -1
*/
void IChain::pointChain(OPoint & p, bool s)
	{
	map->pixelWrite(p.getX(),p.getY(),plab);
	OPoint *ap = new OPoint(p.getX(),p.getY());
	if(s)
		cl->newObject(ap);
	else
		cl->objectInsert(ap,0);
	}

