/**	\file	TreatStructuring.cpp
*	\author	Omar Hamidou
*	\date	08/04
*/

//include
#include <iostream>
#include <deque>
//#include <vector>
#include <algorithm>
#include <math.h>

using namespace std;

#include <GTL\graph.h>
#include <GOMLib\GOMLib.hpp>

#include <LGALib\TreatCMatching.hpp>

#include <LGALib\TreatStructuring.hpp>
/***************************************
	CMatch class
****************************************/

/**
* Constructor
*/
Structured::Structured()
	{
	}

/**
* Destructor
*/
Structured::~Structured()
	{
	clear();
	}

/**
* Clear
*/
void Structured::clear()
	{
	pa.clear();
	qa.clear();
	}


/**********************************************************************
*							Structuration							  *
**********************************************************************/

/**
* Match a OHL of OHL object
* be sure to use parse function before
*/
deque<OGraphic*> Structured::structuring(deque<OHL*> & hae)
	{
	
	OHL *p; deque<OGraphic*> har;
	for(int i=0;i<hae.size();i++)
		{
		if(hae[i]->sameObject((string)"OLine",(string)"OQuadrilateral"))
			{
			og = hae[i]->readOGraphicArray();
			structured();

			p = new OHL();
			p->writeOGraphicArray(pa);
			har.push_back(p);

			clear();
			}
		else
			har.push_back(hae[i]);
		}

	return(har);
	}


/**
* Take the OGraphic list and separate its differents objects
*/
void Structured::separateObject()
{
	for(int i=0;i<og.size();i++)
	{
		if((string)"OLine" == og[i]->getName())
		{
			OLine *l = new OLine();
			l = castOLine(og[i]->clone());
			la.push_back(l);
		}
		if((string)"OQuadrilateral" == og[i]->getName())
		{
			OQuadrilateral *q = new OQuadrilateral();
			q = castOQuadrilateral(og[i]->clone());
			qa.push_back(q);
		}
	}
}

/**
* Transform quarilaterals to graph  
*/
void Structured::ObjectToGraph()
{
	o = new OGraph(false); //the graph is undirected
	for(int i=0;i<og.size();i++)
	{
		if((string)"OLine" == og[i]->getName())
		{
			OLine *l = new OLine();
			l = castOLine(og[i]->clone());
			o->newNode(l);
		}
		if((string)"OQuadrilateral" == og[i]->getName())
		{
			OQuadrilateral *q = new OQuadrilateral();
			q = castOQuadrilateral(og[i]->clone());
			o->newNode(q);
		}
	}
}

/**
* Try to find relations between the different quadrilaterals
*/
void Structured::findRelation()
{
	int pos = la.size();
	int i,j;
	for(i=0;i<qa.size();i++)
	{
		for(j=i+1;j<qa.size();j++)
			if(qa[i]->isConnected(*qa[j]))
				o->newEdge(pos+i,pos+j,new ALabel("QConnected"));
			else 
				if(qa[i]->isTouch(*qa[j]))
					o->newEdge(pos+i,pos+j,new ALabel("Commun_Point"));
	}

	for(i=0;i<la.size();i++)
		for(int j=0;j<qa.size();j++)
			if(qa[j]->isTouch(*la[i]))
				o->newEdge(pos+j,i,new ALabel("Link_QV"));

	for(i=0;i<la.size();i++)
		for(j=i+1;j<la.size();j++)
			if(la[i]->isConnected(*la[j]))
				o->newEdge(i,j,new ALabel("VConnected"));
}


/**
* Structured in order to obtain a structuring graph of quadrilaterals
*/
void Structured::structured()
{
	ObjectToGraph();
	separateObject();
	findRelation();

	pa.push_back(o);
}
