/**	\file	TreatWDPoly.cpp
*	\author	Mathieu Delalandre
*	\date	06/04
*/

//include

#include <iostream>
#include <fstream>
#include <vector>
#include <deque>
#include <math.h>

using namespace std;

#include <GTL\graph.h>
#include <GOMLib\GOMLib.hpp>

#include <LGALib\TreatWDPoly.hpp>


/**
* Constructor
*/
WDP::WDP()
	{
	}


/**
* Destructor
*/
WDP::~WDP()
	{
	pa.clear();
	la.clear();
	}

/**
* Clear
*/
void WDP::clear()
	{
	pa.clear();
	la.clear();
	}
	
/**
* Polygonalise a OHL of OHL object
* be sure to use parse function before
*/
deque<OGraphic*> WDP::polygonalise(deque<OHL*> hae,double th)
	{
	this->th = th;
	
	OHL *p; deque<OGraphic*> har;
	for(int i=0;i<hae.size();i++)
		if(hae[i]->sameObject((string)"OPoint"))
			{
			pa = castOPointArray(hae[i]->readOGraphicArray()); 
			polygonalise(); //pa to la
			if(la.size()>0)
				{
				p = new OHL(); 
				p->writeOGraphicArray(la);
				har.push_back(p);
				}
			else
				har.push_back(hae[i]->clone());
			
			clear();
			}
		else
			har.push_back(hae[i]);

	return har;
	}

/**
* Main Wall & Danielson algorithm 
*/
void WDP::polygonalise()
	{
	OLine *l; int cp=0; 
	bool poly=true; double s;
	if(pa.size()>1)
		while(poly)
			{
			l = new OLine();
			l->setBegin(*pa[cp]);
			for(int i=cp+1;i<pa.size();i++)
				{
				s = surface(cp,i);
				if(s>th)
					{ 
					if(cp == i-1)
						{ cp = i; l->setEnd(*pa[i]); }
					else
						{ cp = i-1; l->setEnd(*pa[i-1]); }
					la.push_back(l);
					break;
					}
				else if(i==pa.size()-1)
					{
					l->setEnd(*pa[i]);
					la.push_back(l);
					poly = false;
					break;
					}
				}
			}
	}


/**
* Compute the surface between a curve and its corresponding vector
*/
double WDP::surface(int b,int e)
	{
	double xd = pa[b]->getX(), yd = pa[b]->getY();
	double dx = pa[b]->getDX(*pa[e]), dy = pa[b]->getDY(*pa[e]);
	double size = e-b; double xi = dx/size, yi = dy/size;
	double s=0; double xp,yp; OVPoint sP, cP;
	for(int i=1;i<=size;i++)
		{
		xp = xd +i*xi; yp = yd +i*yi;
		sP.set(xp,yp);
		cP.set(pa[b+i]->getX(),pa[b+i]->getY());
		s += cP.euclid(sP);
		}
	return s;
	}
	
/**
* Point print (debug)
*/
void WDP::pointPrint(int b,int e)
	{
	for(int i=b;i<e;i++)
		pa[i]->print();
	cout << endl;
	}

/**
* Line print (debug)
*/
void WDP::linePrint(int b,int e)
	{
	for(int i=b;i<e;i++)
		la[i]->print();
	cout << endl;
	}