/**	
*   @file	OGraph.hpp
*	@author	Mathieu Delalandre
*	@date	06/2004	
*/


/**
*  OGraph object
*/
class OGraph : public OGraphic
{
	protected:
		graph og;
		node_map<OGraphic*> nl;	//OGraphic map
		edge_map<AGraphic*> el;	//AGraphic map
	
	public:
		OGraphic* clone() const;
		inline OGraphic* create() const;
		OGraph();
		OGraph(bool);
		~OGraph();

		inline node newNode();
		inline node newNode(OGraphic*);
		inline node nodeRead(int);
		inline int rangeRead(node) const;
		
		inline OGraphic* objectRead(int);
		inline OGraphic* objectRead(node);
		inline int rangeRead(OGraphic*);
		inline void objectWrite(OGraphic*,int);
		inline void objectWrite(OGraphic*,node);
		inline void objectPush(OGraphic*,int);
		inline void objectPush(OGraphic*,node);
		
		inline edge newEdge(int,int);
		inline edge newEdge(int,int,AGraphic*);
		inline edge newEdge(node,node);
		inline edge newEdge(node,node,AGraphic*);
		inline edge edgeRead(int);
		inline int rangeRead(edge) const;

		inline AGraphic* attributeRead(int);
		inline AGraphic* attributeRead(edge);
		inline int rangeRead(AGraphic*);
		inline void attributeWrite(AGraphic*,int);
		inline void attributeWrite(AGraphic*,edge);
		inline void attributePush(AGraphic*,int);
		inline void attributePush(AGraphic*,edge);

		inline deque<OGraphic*> readOGraphicArray();
		deque<OGraphic*> readOGraphicArray(string &);
		void readOGraphicArray(string &,deque<OGraphic*> &);
		
		void writeOGraphicArray(string &,deque<OGraphic*> &);
		int writeOGraphicArray(string &,deque<OGraphic*> &,int);
		
		//to be continue .... read/write attribute array

		void gomo(ostream &);
		void gomi(istream &);
		void xmlo(ostream &);
		void svgo(ostream &);
};

/**
*  Create a new graph object
*/
inline OGraphic* OGraph::create() const
	{ return new OGraph(); }

/**
*  New node with a OGraphicImpl object
*/
inline node OGraph::newNode()
	{ node n = og.new_node(); nl[n] = new OGraphicImpl(); return n; }
/**
*  New node
*/
node OGraph::newNode(OGraphic* o)
	{ node n = og.new_node(); nl[n] = o; return n; }
/**
*  Node read of a given position
*  (0 to size-1)
*/
inline node OGraph::nodeRead(int p)
	{
	graph::node_iterator nit = og.nodes_begin(); int c = 0;
	while((nit != og.nodes_end())&&(c != p))
		{ nit++; c++;}
	return *nit;
	}
/**
*  Range read of given node, or return -1
*/
inline int OGraph::rangeRead(node n) const
	{
	graph::node_iterator nit = og.nodes_begin();
	for(int i=0;i<og.number_of_nodes();i++)
		{ if(*nit == n) return i; nit++; }
	return -1;
	}

/**
*  Object read according to a given position
*  (0 to size-1)
*/
inline OGraphic* OGraph::objectRead(int p)
	{ node n = nodeRead(p); return nl[n]; }
/**
*  Object read according to a read node
*/
inline OGraphic* OGraph::objectRead(node n)
	{ return nl[n]; }
/**
*  Range read according to a read OGraphic, or return -1
*/
inline int OGraph::rangeRead(OGraphic* o)
	{ 
	for(int i=0;i<og.number_of_nodes();i++)
		if(objectRead(i) == o)
			return i;
	return -1;
	}
/**
*  Object write
*  (0 to size-1)
*/
inline void OGraph::objectWrite(OGraphic *o,int p)
	{ node n = nodeRead(p); nl[n] = o; }
/**
*  Object write
*/
inline void OGraph::objectWrite(OGraphic *o,node n)
	{ nl[n] = o; }

/**
*  Object push, delete the p existing object
*  (0 to size-1)
*/
inline void OGraph::objectPush(OGraphic *o,int p)
	{ node n = nodeRead(p); nl[n]->~OGraphic(); nl[n] = o; }
/**
*  Object push, delete the p existing object
*  (0 to size-1)
*/
inline void OGraph::objectPush(OGraphic *o,node n)
	{ nl[n]->~OGraphic(); nl[n] = o; }

/**
*  New edge between two positions with a AGraphicImpl attribute
*  (0 to size-1)
*/
inline edge OGraph::newEdge(int n1, int n2)
	{ edge e = og.new_edge(nodeRead(n1),nodeRead(n2)); el[e] = new AGraphicImpl(); return e; }
/**
*  New edge between two positions 
*  (0 to size-1)
*/
inline edge OGraph::newEdge(int n1, int n2,AGraphic* a)
	{ edge e = og.new_edge(nodeRead(n1),nodeRead(n2)); el[e] = a; return e;}
/**
*  New edge between two nodes with a AGraphicImpl attribute
*/
inline edge OGraph::newEdge(node n1, node n2)
	{ edge e = og.new_edge(n1,n2); el[e] = new AGraphicImpl(); return e; }
/**
*  New edge between two nodes
*/
inline edge OGraph::newEdge(node n1, node n2,AGraphic* a)
	{ edge e = og.new_edge(n1,n2); el[e] = a; return e; }
/**
*  Edge read according to a given position
*  (0 to size-1)
*/
inline edge OGraph::edgeRead(int p)
	{
	graph::edge_iterator eit = og.edges_begin(); int c = 0;
	while((eit != og.edges_end())&&(c != p))
		{ c++; eit++; }
	return *eit;
	}
/**
*  Range read according to a read edge, or return -1
*/
inline int OGraph::rangeRead(edge e) const
	{
	graph::edge_iterator eit = og.edges_begin();
	for(int i=0;i<og.number_of_edges();i++)
		{ if(*eit == e) return i; eit++; }
	return -1;
	}

/**
*  Attribute read according to a given position
*  (0 to size-1)
*/
inline AGraphic* OGraph::attributeRead(int p)
	{ edge e = edgeRead(p); return el[e]; }
/**
*  Attribute read according to a read edge
*/
inline AGraphic* OGraph::attributeRead(edge e)
	{ return el[e]; }
/**
*  Range read according to a read OGraphic, or return -1
*/
inline int OGraph::rangeRead(AGraphic* a)
	{ 
	for(int i=0;i<og.number_of_edges();i++)
		if(attributeRead(i) == a)
			return i;
	return -1;
	}
/**
*  Attribute write according to a given position
*  (0 to size-1)
*/
void OGraph::attributeWrite(AGraphic *a, int p)
	{ edge e = edgeRead(p); el[e] = a;}
/**
*  Attribute write according to a read edge
*/
inline void OGraph::attributeWrite(AGraphic* a, edge e)
	{ 	el[e] = a; }
/**
*  Attribute push according to a given position,
*  delete the p existing attribute
*  (0 to size-1)
*/
void OGraph::attributePush(AGraphic *a, int p)
	{ edge e = edgeRead(p); el[e]->~AGraphic(); el[e] = a;}
/**
*  Attribute push according to a read edge, 
*  delete the p existing attribute
*/
inline void OGraph::attributePush(AGraphic* a, edge e)
	{ el[e]->~AGraphic(); el[e] = a; }
/**
*  Read OGraphic array, extract OGraphic* from nodes
*/
inline deque<OGraphic*> OGraph::readOGraphicArray()
	{
	deque<OGraphic*> oa;
	graph::node_iterator nit = og.nodes_begin();
	while(nit != og.nodes_end())
		{ oa.push_back(nl[*nit]); nit++; }
	return oa;
	}
